# coding=utf-8
# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from knack.help_files import helps  # pylint: disable=unused-import
# pylint: disable=line-too-long, too-many-lines

helps['cloud'] = """
type: group
short-summary: Manage registered Azure clouds.
"""

helps['cloud list'] = """
type: command
short-summary: List registered clouds.
"""

helps['cloud list-profiles'] = """
type: command
short-summary: List the supported profiles for a cloud.
"""

helps['cloud register'] = """
type: command
short-summary: Register a cloud.
long-summary: When registering a cloud, specify only the resource manager endpoint for the autodetection of other endpoints.
examples:
  - name: Register a cloud with a config file
    text: |
        az cloud register -n MyCloud --cloud-config @"cloud.json"
        ("cloud.json" supports all the endpoint and suffix options in camel case or the JSON output format from `az cloud show`. See the example content below.)
        {
          "endpointActiveDirectory": "https://login.microsoftonline.us",
          "suffixAcrLoginServerEndpoint": ".azurecr.us"
        }
        or
        {
          "endpoints":{
            "activeDirectory": "https://login.microsoftonline.us"
          },
          "suffixes":{
            "acrLoginServerEndpoint": ".azurecr.us"
          }
        }
"""

helps['cloud set'] = """
type: command
short-summary: Set the active cloud.
examples:
  - name: Set the active cloud. (autogenerated)
    text: |
        az cloud set --name MyRegisteredCloud
    crafted: true
"""

helps['cloud show'] = """
type: command
short-summary: Get the details of a registered cloud.
"""

helps['cloud unregister'] = """
type: command
short-summary: Unregister a cloud.
examples:
  - name: Unregister a cloud. (autogenerated)
    text: |
        az cloud unregister --name MyRegisteredCloud
    crafted: true
"""

helps['cloud update'] = """
type: command
short-summary: Update the configuration of a cloud.
examples:
  - name: Update the configuration of a cloud. (autogenerated)
    text: |
        az cloud update --profile latest
    crafted: true
"""



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
#
# Code generated by aaz-dev-tools
# --------------------------------------------------------------------------------------------

# pylint: skip-file
# flake8: noqa

from azure.cli.core.aaz import *


@register_command(
    "network vnet peering update",
)
class Update(AAZCommand):
    """Update a peering.

    :example: Change forwarded traffic configuration of a virtual network peering.
        az network vnet peering update -g MyResourceGroup -n MyVnet1ToMyVnet2 --vnet-name MyVnet1 --set allowForwardedTraffic=true

    :example: Change virtual network access of a virtual network peering.
        az network vnet peering update -g MyResourceGroup -n MyVnet1ToMyVnet2 --vnet-name MyVnet1 --set allowVirtualNetworkAccess=true

    :example: Change gateway transit property configuration of a virtual network peering.
        az network vnet peering update -g MyResourceGroup -n MyVnet1ToMyVnet2 --vnet-name MyVnet1 --set allowGatewayTransit=true

    :example: Use remote gateways in virtual network peering.
        az network vnet peering update -g MyResourceGroup -n MyVnet1ToMyVnet2 --vnet-name MyVnet1 --set useRemoteGateways=true
    """

    _aaz_info = {
        "version": "2022-01-01",
        "resources": [
            ["mgmt-plane", "/subscriptions/{}/resourcegroups/{}/providers/microsoft.network/virtualnetworks/{}/virtualnetworkpeerings/{}", "2022-01-01"],
        ]
    }

    AZ_SUPPORT_NO_WAIT = True

    AZ_SUPPORT_GENERIC_UPDATE = True

    def _handler(self, command_args):
        super()._handler(command_args)
        return self.build_lro_poller(self._execute_operations, self._output)

    _args_schema = None

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        if cls._args_schema is not None:
            return cls._args_schema
        cls._args_schema = super()._build_arguments_schema(*args, **kwargs)

        # define Arg Group ""

        _args_schema = cls._args_schema
        _args_schema.resource_group = AAZResourceGroupNameArg(
            required=True,
        )
        _args_schema.vnet_name = AAZStrArg(
            options=["--vnet-name"],
            help="The virtual network (VNet) name.",
            required=True,
            id_part="name",
        )
        _args_schema.name = AAZStrArg(
            options=["-n", "--name"],
            help="The name of the VNet peering.",
            required=True,
            id_part="child_name_1",
        )

        # define Arg Group "Properties"

        # define Arg Group "VirtualNetworkPeeringParameters"
        return cls._args_schema

    _args_address_space_update = None

    @classmethod
    def _build_args_address_space_update(cls, _schema):
        if cls._args_address_space_update is not None:
            _schema.address_prefixes = cls._args_address_space_update.address_prefixes
            return

        cls._args_address_space_update = AAZObjectArg(
            nullable=True,
        )

        address_space_update = cls._args_address_space_update
        address_space_update.address_prefixes = AAZListArg(
            options=["address-prefixes"],
            help="A list of address blocks reserved for this virtual network in CIDR notation.",
            nullable=True,
        )

        address_prefixes = cls._args_address_space_update.address_prefixes
        address_prefixes.Element = AAZStrArg(
            nullable=True,
        )

        _schema.address_prefixes = cls._args_address_space_update.address_prefixes

    def _execute_operations(self):
        self.pre_operations()
        self.VirtualNetworkPeeringsGet(ctx=self.ctx)()
        self.pre_instance_update(self.ctx.vars.instance)
        self.InstanceUpdateByJson(ctx=self.ctx)()
        self.InstanceUpdateByGeneric(ctx=self.ctx)()
        self.post_instance_update(self.ctx.vars.instance)
        yield self.VirtualNetworkPeeringsCreateOrUpdate(ctx=self.ctx)()
        self.post_operations()

    @register_callback
    def pre_operations(self):
        pass

    @register_callback
    def post_operations(self):
        pass

    @register_callback
    def pre_instance_update(self, instance):
        pass

    @register_callback
    def post_instance_update(self, instance):
        pass

    def _output(self, *args, **kwargs):
        result = self.deserialize_output(self.ctx.vars.instance, client_flatten=True)
        return result

    class VirtualNetworkPeeringsGet(AAZHttpOperation):
        CLIENT_TYPE = "MgmtClient"

        def __call__(self, *args, **kwargs):
            request = self.make_request()
            session = self.client.send_request(request=request, stream=False, **kwargs)
            if session.http_response.status_code in [200]:
                return self.on_200(session)

            return self.on_error(session.http_response)

        @property
        def url(self):
            return self.client.format_url(
                "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Network/virtualNetworks/{virtualNetworkName}/virtualNetworkPeerings/{virtualNetworkPeeringName}",
                **self.url_parameters
            )

        @property
        def method(self):
            return "GET"

        @property
        def error_format(self):
            return "ODataV4Format"

        @property
        def url_parameters(self):
            parameters = {
                **self.serialize_url_param(
                    "resourceGroupName", self.ctx.args.resource_group,
                    required=True,
                ),
                **self.serialize_url_param(
                    "subscriptionId", self.ctx.subscription_id,
                    required=True,
                ),
                **self.serialize_url_param(
                    "virtualNetworkName", self.ctx.args.vnet_name,
                    required=True,
                ),
                **self.serialize_url_param(
                    "virtualNetworkPeeringName", self.ctx.args.name,
                    required=True,
                ),
            }
            return parameters

        @property
        def query_parameters(self):
            parameters = {
                **self.serialize_query_param(
                    "api-version", "2022-01-01",
                    required=True,
                ),
            }
            return parameters

        @property
        def header_parameters(self):
            parameters = {
                **self.serialize_header_param(
                    "Accept", "application/json",
                ),
            }
            return parameters

        def on_200(self, session):
            data = self.deserialize_http_content(session)
            self.ctx.set_var(
                "instance",
                data,
                schema_builder=self._build_schema_on_200
            )

        _schema_on_200 = None

        @classmethod
        def _build_schema_on_200(cls):
            if cls._schema_on_200 is not None:
                return cls._schema_on_200

            cls._schema_on_200 = AAZObjectType()
            _UpdateHelper._build_schema_virtual_network_peering_read(cls._schema_on_200)

            return cls._schema_on_200

    class VirtualNetworkPeeringsCreateOrUpdate(AAZHttpOperation):
        CLIENT_TYPE = "MgmtClient"

        def __call__(self, *args, **kwargs):
            request = self.make_request()
            session = self.client.send_request(request=request, stream=False, **kwargs)
            if session.http_response.status_code in [202]:
                return self.client.build_lro_polling(
                    self.ctx.args.no_wait,
                    session,
                    self.on_200_201,
                    self.on_error,
                    lro_options={"final-state-via": "azure-async-operation"},
                    path_format_arguments=self.url_parameters,
                )
            if session.http_response.status_code in [200, 201]:
                return self.client.build_lro_polling(
                    self.ctx.args.no_wait,
                    session,
                    self.on_200_201,
                    self.on_error,
                    lro_options={"final-state-via": "azure-async-operation"},
                    path_format_arguments=self.url_parameters,
                )

            return self.on_error(session.http_response)

        @property
        def url(self):
            return self.client.format_url(
                "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Network/virtualNetworks/{virtualNetworkName}/virtualNetworkPeerings/{virtualNetworkPeeringName}",
                **self.url_parameters
            )

        @property
        def method(self):
            return "PUT"

        @property
        def error_format(self):
            return "ODataV4Format"

        @property
        def url_parameters(self):
            parameters = {
                **self.serialize_url_param(
                    "resourceGroupName", self.ctx.args.resource_group,
                    required=True,
                ),
                **self.serialize_url_param(
                    "subscriptionId", self.ctx.subscription_id,
                    required=True,
                ),
                **self.serialize_url_param(
                    "virtualNetworkName", self.ctx.args.vnet_name,
                    required=True,
                ),
                **self.serialize_url_param(
                    "virtualNetworkPeeringName", self.ctx.args.name,
                    required=True,
                ),
            }
            return parameters

        @property
        def query_parameters(self):
            parameters = {
                **self.serialize_query_param(
                    "api-version", "2022-01-01",
                    required=True,
                ),
            }
            return parameters

        @property
        def header_parameters(self):
            parameters = {
                **self.serialize_header_param(
                    "Content-Type", "application/json",
                ),
                **self.serialize_header_param(
                    "Accept", "application/json",
                ),
            }
            return parameters

        @property
        def content(self):
            _content_value, _builder = self.new_content_builder(
                self.ctx.args,
                value=self.ctx.vars.instance,
            )

            return self.serialize_content(_content_value)

        def on_200_201(self, session):
            data = self.deserialize_http_content(session)
            self.ctx.set_var(
                "instance",
                data,
                schema_builder=self._build_schema_on_200_201
            )

        _schema_on_200_201 = None

        @classmethod
        def _build_schema_on_200_201(cls):
            if cls._schema_on_200_201 is not None:
                return cls._schema_on_200_201

            cls._schema_on_200_201 = AAZObjectType()
            _UpdateHelper._build_schema_virtual_network_peering_read(cls._schema_on_200_201)

            return cls._schema_on_200_201

    class InstanceUpdateByJson(AAZJsonInstanceUpdateOperation):

        def __call__(self, *args, **kwargs):
            self._update_instance(self.ctx.vars.instance)

        def _update_instance(self, instance):
            _instance_value, _builder = self.new_content_builder(
                self.ctx.args,
                value=instance,
                typ=AAZObjectType
            )
            _builder.set_prop("name", AAZStrType, ".name")
            _builder.set_prop("properties", AAZObjectType, typ_kwargs={"flags": {"client_flatten": True}})

            properties = _builder.get(".properties")
            if properties is not None:
                properties.set_prop("remoteVirtualNetwork", AAZObjectType)

            return _instance_value

    class InstanceUpdateByGeneric(AAZGenericInstanceUpdateOperation):

        def __call__(self, *args, **kwargs):
            self._update_instance_by_generic(
                self.ctx.vars.instance,
                self.ctx.generic_update_args
            )


class _UpdateHelper:
    """Helper class for Update"""

    @classmethod
    def _build_schema_address_space_update(cls, _builder):
        if _builder is None:
            return
        _builder.set_prop("addressPrefixes", AAZListType, ".address_prefixes")

        address_prefixes = _builder.get(".addressPrefixes")
        if address_prefixes is not None:
            address_prefixes.set_elements(AAZStrType, ".")

    _schema_address_space_read = None

    @classmethod
    def _build_schema_address_space_read(cls, _schema):
        if cls._schema_address_space_read is not None:
            _schema.address_prefixes = cls._schema_address_space_read.address_prefixes
            return

        cls._schema_address_space_read = _schema_address_space_read = AAZObjectType()

        address_space_read = _schema_address_space_read
        address_space_read.address_prefixes = AAZListType(
            serialized_name="addressPrefixes",
        )

        address_prefixes = _schema_address_space_read.address_prefixes
        address_prefixes.Element = AAZStrType()

        _schema.address_prefixes = cls._schema_address_space_read.address_prefixes

    _schema_virtual_network_peering_read = None

    @classmethod
    def _build_schema_virtual_network_peering_read(cls, _schema):
        if cls._schema_virtual_network_peering_read is not None:
            _schema.etag = cls._schema_virtual_network_peering_read.etag
            _schema.id = cls._schema_virtual_network_peering_read.id
            _schema.name = cls._schema_virtual_network_peering_read.name
            _schema.properties = cls._schema_virtual_network_peering_read.properties
            _schema.type = cls._schema_virtual_network_peering_read.type
            return

        cls._schema_virtual_network_peering_read = _schema_virtual_network_peering_read = AAZObjectType()

        virtual_network_peering_read = _schema_virtual_network_peering_read
        virtual_network_peering_read.etag = AAZStrType(
            flags={"read_only": True},
        )
        virtual_network_peering_read.id = AAZStrType()
        virtual_network_peering_read.name = AAZStrType()
        virtual_network_peering_read.properties = AAZObjectType(
            flags={"client_flatten": True},
        )
        virtual_network_peering_read.type = AAZStrType()

        properties = _schema_virtual_network_peering_read.properties
        properties.allow_forwarded_traffic = AAZBoolType(
            serialized_name="allowForwardedTraffic",
        )
        properties.allow_gateway_transit = AAZBoolType(
            serialized_name="allowGatewayTransit",
        )
        properties.allow_virtual_network_access = AAZBoolType(
            serialized_name="allowVirtualNetworkAccess",
        )
        properties.do_not_verify_remote_gateways = AAZBoolType(
            serialized_name="doNotVerifyRemoteGateways",
        )
        properties.peering_state = AAZStrType(
            serialized_name="peeringState",
        )
        properties.peering_sync_level = AAZStrType(
            serialized_name="peeringSyncLevel",
        )
        properties.provisioning_state = AAZStrType(
            serialized_name="provisioningState",
            flags={"read_only": True},
        )
        properties.remote_address_space = AAZObjectType(
            serialized_name="remoteAddressSpace",
        )
        cls._build_schema_address_space_read(properties.remote_address_space)
        properties.remote_bgp_communities = AAZObjectType(
            serialized_name="remoteBgpCommunities",
        )
        properties.remote_virtual_network = AAZObjectType(
            serialized_name="remoteVirtualNetwork",
        )
        properties.remote_virtual_network_address_space = AAZObjectType(
            serialized_name="remoteVirtualNetworkAddressSpace",
        )
        cls._build_schema_address_space_read(properties.remote_virtual_network_address_space)
        properties.remote_virtual_network_encryption = AAZObjectType(
            serialized_name="remoteVirtualNetworkEncryption",
        )
        properties.resource_guid = AAZStrType(
            serialized_name="resourceGuid",
            flags={"read_only": True},
        )
        properties.use_remote_gateways = AAZBoolType(
            serialized_name="useRemoteGateways",
        )

        remote_bgp_communities = _schema_virtual_network_peering_read.properties.remote_bgp_communities
        remote_bgp_communities.regional_community = AAZStrType(
            serialized_name="regionalCommunity",
            flags={"read_only": True},
        )
        remote_bgp_communities.virtual_network_community = AAZStrType(
            serialized_name="virtualNetworkCommunity",
            flags={"required": True},
        )

        remote_virtual_network = _schema_virtual_network_peering_read.properties.remote_virtual_network
        remote_virtual_network.id = AAZStrType()

        remote_virtual_network_encryption = _schema_virtual_network_peering_read.properties.remote_virtual_network_encryption
        remote_virtual_network_encryption.enabled = AAZBoolType(
            flags={"required": True},
        )
        remote_virtual_network_encryption.enforcement = AAZStrType()

        _schema.etag = cls._schema_virtual_network_peering_read.etag
        _schema.id = cls._schema_virtual_network_peering_read.id
        _schema.name = cls._schema_virtual_network_peering_read.name
        _schema.properties = cls._schema_virtual_network_peering_read.properties
        _schema.type = cls._schema_virtual_network_peering_read.type


__all__ = ["Update"]



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from azure.cli.core.azclierror import AzCLIError

from azure.cli.command_modules.acs._client_factory import get_resource_groups_client


def get_rg_location(ctx, resource_group_name, subscription_id=None):
    groups = get_resource_groups_client(ctx, subscription_id=subscription_id)
    # Just do the get, we don't need the result, it will error out if the group doesn't exist.
    rg = groups.get(resource_group_name)
    if rg is None:
        raise AzCLIError(f"Resource group {resource_group_name} not found.")
    return rg.location



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from knack.log import get_logger
from knack.util import CLIError
from azure.cli.command_modules.redis._client_factory import cf_redis

logger = get_logger(__name__)

allowed_c_family_sizes = ['c0', 'c1', 'c2', 'c3', 'c4', 'c5', 'c6']
allowed_p_family_sizes = ['p1', 'p2', 'p3', 'p4', 'p5']
wrong_vmsize_error = CLIError('Invalid VM size. Example for Valid values: '
                              'For Standard Sku : (C0, C1, C2, C3, C4, C5, C6), '
                              'for Premium Sku : (P1, P2, P3, P4, P5)')
allowed_auth_methods = ['SAS', 'ManagedIdentity']
# region Custom Commands


# pylint: disable=unused-argument
def cli_redis_export(cmd, client, resource_group_name, name, prefix, container,
                     preferred_data_archive_auth_method=None, file_format=None):
    from azure.mgmt.redis.models import ExportRDBParameters
    parameters = ExportRDBParameters(prefix=prefix, container=container, format=file_format,
                                     preferred_data_archive_auth_method=preferred_data_archive_auth_method)
    return client.begin_export_data(resource_group_name, name, parameters)


# pylint: disable=unused-argument
def cli_redis_update(cmd, instance, sku=None, vm_size=None):
    from azure.mgmt.redis.models import RedisUpdateParameters
    if sku is not None:
        instance.sku.name = sku

    if vm_size is not None:
        instance.sku.family = vm_size[0]
        instance.sku.capacity = vm_size[1:]

    # avoid setting memory configs for basic sku
    if instance.sku.name == 'Basic':
        memory_configs = ['maxmemory-reserved', 'maxfragmentationmemory-reserved', 'maxmemory-delta']
        instance.redis_configuration.maxmemory_reserved = None
        instance.redis_configuration.maxfragmentationmemory_reserved = None
        instance.redis_configuration.maxmemory_delta = None
        properties = instance.redis_configuration.additional_properties
        for memory_config in memory_configs:
            if properties is not None and memory_config in properties:
                instance.redis_configuration.additional_properties.pop(memory_config, None)
    # trim RDB and AOF connection strings and zonal-configuration
    removable_keys = [
        'rdb-storage-connection-string',
        'aof-storage-connection-string-0',
        'aof-storage-connection-string-1',
        'zonal-configuration']
    instance.redis_configuration.rdb_storage_connection_string = None
    instance.redis_configuration.aof_storage_connection_string0 = None
    instance.redis_configuration.aof_storage_connection_string1 = None
    properties = instance.redis_configuration.additional_properties
    for key in removable_keys:
        if properties is not None and key in properties:
            instance.redis_configuration.additional_properties.pop(key, None)
    # pylint: disable=too-many-function-args
    update_params = RedisUpdateParameters(
        redis_configuration=instance.redis_configuration,
        enable_non_ssl_port=instance.enable_non_ssl_port,
        tenant_settings=instance.tenant_settings,
        shard_count=instance.shard_count,
        minimum_tls_version=instance.minimum_tls_version,
        redis_version=instance.redis_version,
        sku=instance.sku,
        tags=instance.tags
    )
    return update_params


def custom_update_setter(client, resource_group_name, name, parameters):
    # Custom update setter is used to match behavior from when update was not a LRO
    return client.begin_update(resource_group_name, name, parameters).result(0)


# pylint: disable=unused-argument
# pylint: disable=too-many-locals
def cli_redis_create(cmd, client,
                     resource_group_name, name, location, sku, vm_size, tags=None,
                     redis_configuration=None, enable_non_ssl_port=None, tenant_settings=None,
                     shard_count=None, minimum_tls_version=None, subnet_id=None, static_ip=None,
                     zones=None, replicas_per_master=None, redis_version=None, mi_system_assigned=None,
                     mi_user_assigned=None):
    # pylint:disable=line-too-long
    if ((sku.lower() in ['standard', 'basic'] and vm_size.lower() not in allowed_c_family_sizes) or (sku.lower() in ['premium'] and vm_size.lower() not in allowed_p_family_sizes)):
        raise wrong_vmsize_error
    tenant_settings_in_json = {}
    if tenant_settings is not None:
        for item in tenant_settings:
            tenant_settings_in_json.update(get_key_value_pair(item))
    from azure.mgmt.redis.models import RedisCreateParameters, Sku, RedisCommonPropertiesRedisConfiguration
    identity = build_identity(mi_system_assigned, mi_user_assigned)
    if identity.type == "None":
        identity = None
    # pylint: disable=too-many-function-args
    params = RedisCreateParameters(
        sku=Sku(name=sku, family=vm_size[0], capacity=vm_size[1:]),
        location=location,
        redis_configuration=RedisCommonPropertiesRedisConfiguration(additional_properties=redis_configuration),
        enable_non_ssl_port=enable_non_ssl_port,
        replicas_per_master=replicas_per_master,
        tenant_settings=tenant_settings_in_json,
        shard_count=shard_count,
        minimum_tls_version=minimum_tls_version,
        subnet_id=subnet_id,
        static_ip=static_ip,
        zones=zones,
        redis_version=redis_version,
        identity=identity,
        public_network_access=None,
        tags=tags)
    return client.begin_create(resource_group_name, name, params)


def get_key_value_pair(string):
    result = {}
    if string:
        kvp = string.split('=', 1)
        result = {kvp[0]: kvp[1]} if len(kvp) > 1 else {string: ''}
    return result


# pylint: disable=unused-argument
def cli_redis_create_server_link(cmd, client, resource_group_name, name, server_to_link, replication_role):
    redis_client = cf_redis(cmd.cli_ctx)
    from azure.cli.core.commands.client_factory import get_subscription_id
    from msrestazure.tools import is_valid_resource_id, resource_id
    if not is_valid_resource_id(server_to_link):
        server_to_link = resource_id(
            subscription=get_subscription_id(cmd.cli_ctx),
            resource_group=resource_group_name,
            namespace='Microsoft.Cache', type='Redis',
            name=server_to_link
        )

    cache_to_link = get_cache_from_resource_id(redis_client, server_to_link)

    from azure.mgmt.redis.models import RedisLinkedServerCreateParameters
    params = RedisLinkedServerCreateParameters(linked_redis_cache_id=cache_to_link.id,
                                               linked_redis_cache_location=cache_to_link.location,
                                               server_role=replication_role)
    return client.begin_create(resource_group_name, name, cache_to_link.name, params)


def cli_redis_patch_schedule_create_or_update(client, resource_group_name, name, schedule_entries):
    from azure.mgmt.redis.models import RedisPatchSchedule
    param = RedisPatchSchedule(schedule_entries=schedule_entries)
    return client.create_or_update(resource_group_name, name, "default", param)


def cli_redis_patch_schedule_get(client, resource_group_name, name):
    return client.get(resource_group_name, name, "default")


def cli_redis_patch_schedule_delete(client, resource_group_name, name):
    return client.delete(resource_group_name, name, "default")


def cli_redis_list_cache(client, resource_group_name=None):
    cache_list = client.list_by_resource_group(resource_group_name=resource_group_name) \
        if resource_group_name else client.list_by_subscription()
    return list(cache_list)


def get_cache_from_resource_id(client, cache_resource_id):
    from msrestazure.tools import parse_resource_id
    id_comps = parse_resource_id(cache_resource_id)
    return client.get(id_comps['resource_group'], id_comps['name'])


def cli_redis_firewall_create(client, resource_group_name, name, rule_name, start_ip, end_ip):
    from azure.mgmt.redis.models import RedisFirewallRule
    param = RedisFirewallRule(start_ip=start_ip, end_ip=end_ip)
    return client.create_or_update(resource_group_name, name, rule_name, param)


def cli_redis_regenerate_key(client, resource_group_name, name, key_type):
    from azure.mgmt.redis.models import RedisRegenerateKeyParameters
    return client.regenerate_key(resource_group_name, name, RedisRegenerateKeyParameters(key_type=key_type))


def cli_redis_import(client, resource_group_name, name, files,
                     preferred_data_archive_auth_method=None, file_format=None):
    from azure.mgmt.redis.models import ImportRDBParameters
    return client.begin_import_data(resource_group_name, name, ImportRDBParameters(files=files, format=file_format,
                                    preferred_data_archive_auth_method=preferred_data_archive_auth_method))


def cli_redis_force_reboot(client, resource_group_name, name, reboot_type, shard_id=None):
    from azure.mgmt.redis.models import RedisRebootParameters
    param = RedisRebootParameters(reboot_type=reboot_type, shard_id=shard_id)
    return client.force_reboot(resource_group_name, name, param)


def cli_redis_identity_show(client, resource_group_name, cache_name):
    from azure.mgmt.redis.models import ManagedServiceIdentityType, ManagedServiceIdentity
    redis_resourse = client.get(resource_group_name, cache_name)
    if redis_resourse.identity is None:
        redis_resourse.identity = ManagedServiceIdentity(
            type=ManagedServiceIdentityType.NONE.value,
        )
    return redis_resourse.identity


def cli_redis_identity_assign(client, resource_group_name, cache_name, mi_system_assigned=None, mi_user_assigned=None):
    from azure.mgmt.redis.models import RedisUpdateParameters, ManagedServiceIdentityType
    redis_resourse = client.get(resource_group_name, cache_name)
    identity = redis_resourse.identity
    if identity is not None:
        if ManagedServiceIdentityType.SYSTEM_ASSIGNED.value in identity.type:
            mi_system_assigned = True
        if ManagedServiceIdentityType.USER_ASSIGNED.value in identity.type:
            old_user_identity = list(identity.user_assigned_identities)
            if mi_user_assigned is None:
                mi_user_assigned = []
            for user_id in old_user_identity:
                mi_user_assigned.append(user_id)
    update_params = RedisUpdateParameters(
        identity=build_identity(mi_system_assigned, mi_user_assigned))
    redis_resourse = client.begin_update(resource_group_name, cache_name, update_params).result()
    return redis_resourse.identity


def cli_redis_identity_remove(client, resource_group_name, cache_name, mi_system_assigned=None, mi_user_assigned=None):
    from azure.mgmt.redis.models import RedisUpdateParameters, ManagedServiceIdentityType, ManagedServiceIdentity
    redis_resourse = client.get(resource_group_name, cache_name)
    identity = redis_resourse.identity
    system_assigned = None
    none_identity = ManagedServiceIdentity(
        type=ManagedServiceIdentityType.NONE.value)
    if identity is None:
        return none_identity
    if (identity.type == ManagedServiceIdentityType.SYSTEM_ASSIGNED_USER_ASSIGNED.value or
            identity.type == ManagedServiceIdentityType.SYSTEM_ASSIGNED.value):
        system_assigned = True
    if mi_system_assigned is not None:
        system_assigned = None
    user_assigned = None
    if identity.user_assigned_identities is not None:
        user_assigned = list(identity.user_assigned_identities)
    if mi_user_assigned is not None and user_assigned is not None:
        for mi_user_id in mi_user_assigned:
            try:
                user_assigned.remove(mi_user_id)
            except ValueError:
                pass
        if len(user_assigned) == 0:
            user_assigned = None
    update_params = RedisUpdateParameters(
        identity=build_identity(system_assigned, user_assigned)
    )
    updated_resourse = client.begin_update(resource_group_name, cache_name, update_params).result()
    if updated_resourse.identity is None:
        updated_resourse.identity = none_identity
    return updated_resourse.identity


def build_identity(mi_system_assigned, mi_user_assigned):
    from azure.mgmt.redis.models import ManagedServiceIdentity, ManagedServiceIdentityType, UserAssignedIdentity
    identityType = ManagedServiceIdentityType.NONE
    userIdentities = None
    if mi_system_assigned is not None:
        identityType = ManagedServiceIdentityType.SYSTEM_ASSIGNED
    if mi_user_assigned is not None and len(mi_user_assigned) > 0:
        userIdentities = {id: UserAssignedIdentity() for id in mi_user_assigned}
        if identityType == ManagedServiceIdentityType.NONE:
            identityType = ManagedServiceIdentityType.USER_ASSIGNED
        else:
            identityType = ManagedServiceIdentityType.SYSTEM_ASSIGNED_USER_ASSIGNED
    return ManagedServiceIdentity(
        type=identityType.value,
        user_assigned_identities=userIdentities)

# endregion



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
# pylint: disable=line-too-long
# pylint: disable=unused-variable

import re

from datetime import timedelta
from isodate import parse_duration
from knack.util import CLIError

# PARAMETER VALIDATORS
# Type ISO 8061 duration

iso8601pattern = re.compile("^P(?!$)(\\d+Y)?(\\d+M)?(\\d+W)?(\\d+D)?(T(?=\\d)(\\d+H)?(\\d+M)?(\\d+.)?(\\d+S)?)?$")
timedeltapattern = re.compile("^\\d+:\\d+:\\d+$")


def _validate_lock_duration(namespace):
    if namespace.lock_duration:
        if iso8601pattern.match(namespace.lock_duration):
            if parse_duration(namespace.lock_duration) > timedelta(days=0, minutes=6, seconds=0):
                raise CLIError(
                    '--lock-duration Value Error : {0} value, The maximum value for LockDuration is 5 minutes; the default value is 1 minute.'.format(
                        namespace.lock_duration))
        elif timedeltapattern.match(namespace.lock_duration):
            day, miniute, seconds = namespace.lock_duration.split(":")
            if int(day) > 0 or int(miniute) > 6:
                raise CLIError(
                    '--lock-duration Value Error : {0} value, The maximum value for LockDuration is 5 minutes; the default value is 1 minute.'.format(
                        namespace.lock_duration))
        else:
            raise CLIError('--lock-duration Value Error : {0} value is not in ISO 8601 timespan / duration format. e.g.'
                           ' PT10M for duration of 10 min or 00:10:00 for duration of 10 min'.format(namespace.lock_duration))


def _validate_default_message_time_to_live(namespace):
    if namespace.default_message_time_to_live:
        if not iso8601pattern.match(namespace.default_message_time_to_live) and not timedeltapattern.match(namespace.default_message_time_to_live):
            raise CLIError('--default-message-time-to-live Value Error : {0} value is not in ISO 8601 timespan / duration format. e.g. PT10M for duration of 10 min or 00:10:00 for duration of 10 min'.format(namespace.default_message_time_to_live))


def _validate_duplicate_detection_history_time_window(namespace):
    if namespace.duplicate_detection_history_time_window:
        if iso8601pattern.match(namespace.duplicate_detection_history_time_window):
            pass
        elif timedeltapattern.match(namespace.duplicate_detection_history_time_window):
            pass
        else:
            raise CLIError('--duplicate-detection-history-time-window Value Error : {0} value is not in ISO 8601 timespan / duration format. e.g. PT10M for duration of 10 min or 00:10:00 for duration of 10 min'.format(namespace.duplicate_detection_history_time_window))


def _validate_auto_delete_on_idle(namespace):
    if namespace.auto_delete_on_idle:
        if iso8601pattern.match(namespace.auto_delete_on_idle):
            pass
        elif timedeltapattern.match(namespace.auto_delete_on_idle):
            pass
        else:
            raise CLIError('--auto-delete-on-idle Value Error : {0} value is not in ISO 8601 timespan / duration format. e.g. PT10M for duration of 10 min or 00:10:00 for duration of 10 min'.format(namespace.auto_delete_on_idle))


def validate_partner_namespace(cmd, namespace):
    from azure.cli.core.commands.client_factory import get_subscription_id
    from msrestazure.tools import is_valid_resource_id, resource_id
    if namespace.partner_namespace:
        if not is_valid_resource_id(namespace.partner_namespace):
            namespace.partner_namespace = resource_id(
                subscription=get_subscription_id(cmd.cli_ctx),
                resource_group=namespace.resource_group_name,
                namespace='Microsoft.ServiceBus',
                type='namespaces',
                name=namespace.partner_namespace)


def validate_target_namespace(cmd, namespace):
    from azure.cli.core.commands.client_factory import get_subscription_id
    from msrestazure.tools import is_valid_resource_id, resource_id
    if namespace.target_namespace:
        if not is_valid_resource_id(namespace.target_namespace):
            namespace.target_namespace = resource_id(
                subscription=get_subscription_id(cmd.cli_ctx),
                resource_group=namespace.resource_group_name,
                namespace='Microsoft.ServiceBus',
                type='namespaces',
                name=namespace.target_namespace)


def validate_premiumsku_capacity(namespace):
    if namespace.sku and namespace.sku != 'Premium' and namespace.capacity:
        raise CLIError('--capacity - This property is only applicable to namespaces of Premium SKU')


# Validates if a subnet id or name have been given by the user. If subnet id is given, vnet-name should not be provided.
def validate_subnet(cmd, namespace):
    from msrestazure.tools import resource_id, is_valid_resource_id
    from azure.cli.core.commands.client_factory import get_subscription_id

    subnet = namespace.subnet
    subnet_is_id = is_valid_resource_id(subnet)
    vnet = namespace.vnet_name

    if (subnet_is_id and not vnet) or (not subnet and not vnet):
        pass
    elif subnet and not subnet_is_id and vnet:
        namespace.subnet = resource_id(
            subscription=get_subscription_id(cmd.cli_ctx),
            resource_group=namespace.resource_group_name,
            namespace='Microsoft.Network',
            type='virtualNetworks',
            name=vnet,
            child_type_1='subnets',
            child_name_1=subnet)
    else:
        raise CLIError('incorrect usage: [--subnet ID | --subnet NAME --vnet-name NAME]')
    delattr(namespace, 'vnet_name')


def validate_rights(namespace):
    if 'Manage' in namespace.rights:
        if 'Listen' not in namespace.rights or 'Send' not in namespace.rights:
            raise CLIError('Error : Assigning \'Manage\' to --rights requires \'Listen\' and \'Send\' to be included with. e.g. --rights Manage Send Listen')


def validate_private_endpoint_connection_id(namespace):
    from azure.cli.core.azclierror import RequiredArgumentMissingError
    if namespace.connection_id:
        from azure.cli.core.util import parse_proxy_resource_id
        result = parse_proxy_resource_id(namespace.connection_id)
        namespace.resource_group_name = result['resource_group']
        namespace.namespace_name = result['name']
        namespace.private_endpoint_connection_name = result.get('child_name_1')

#    if namespace.account_name and not namespace.resource_group_name:
#        namespace.resource_group_name = _query_account_rg(cmd.cli_ctx, namespace.account_name)[0]

    if not all([namespace.namespace_name, namespace.resource_group_name, namespace.private_endpoint_connection_name]):
        raise RequiredArgumentMissingError("Please provide either `--Id` or `-g` value `--namespace-name` vaule `--name` value")

    del namespace.connection_id
